TopoART <- function() {
	
#--------------------------------------------------------------------------------------------------
	
	setClass("TopoARTEdge",
		representation(
			from="numeric",
			to="numeric"
		),
		prototype(
			from=-1,
			to=-1
		)
	)
	
	setMethod("initialize","TopoARTEdge", function(.Object,src_file,node_index)
	{
		#tmpstring<-sprintf("load TopoART edge %i",node_index-1)
		#print(tmpstring)
		
		f=src_file
		
		l<-readLines(f,n=1)
		substrings<-strsplit(l,split="->")
		num_string<-substrings[[1]][1]
		from<-as.numeric(num_string)
		num_string<-substrings[[1]][2]
		to<-as.numeric(num_string)
		#tmpstring<-sprintf("%i->%i",from,to)
		#print(tmpstring)
		
		.Object@from<-from
		.Object@to<-to
		.Object
	})
	
#--------------------------------------------------------------------------------------------------
	
	setClass("TopoARTNode",
		representation(
			ID="numeric",
			cluster_ID="numeric",
			dim="numeric",
			represented_inputs="numeric",
			activation="numeric",
			match_value="numeric",
			edge_num="numeric",
			edges="vector",
			weights="vector"
		),
		prototype(
			ID=0,
			cluster_ID=0,
			dim=0,
			represented_inputs=0,
			activation=0,
			match_value=0,
			edge_num=0,
			edges=vector(mode="list",length=0),
			weights=vector(mode="list",length=0)
		)
	)
	
	setMethod("initialize","TopoARTNode", function(.Object,src_file,node_index)
	{
		#tmpstring<-sprintf("load TopoART node %i",node_index-1)
		#print(tmpstring)
		
		f=src_file
		l<-readLines(f,n=1)
		
		l<-readLines(f,n=1)
		substrings<-strsplit(l,split=" ")
		num_string<-substrings[[1]][2]
		ID<-as.numeric(num_string)
		#tmpstring<-sprintf("ID: %i",ID)
		#print(tmpstring)
		
		l<-readLines(f,n=1)
		substrings<-strsplit(l,split=" ")
		num_string<-substrings[[1]][2]
		cluster_ID<-as.numeric(num_string)
		#tmpstring<-sprintf("cluster_ID: %i",cluster_ID)
		#print(tmpstring)
		
		l<-readLines(f,n=1)
		substrings<-strsplit(l,split=" ")
		num_string<-substrings[[1]][2]
		dim<-as.numeric(num_string)
		#tmpstring<-sprintf("dim: %i",dim)
		#print(tmpstring)
		
		l<-readLines(f,n=1)
		substrings<-strsplit(l,split=" ")
		num_string<-substrings[[1]][2]
		represented_inputs<-as.numeric(num_string)
		#tmpstring<-sprintf("represented_inputs: %i",represented_inputs)
		#print(tmpstring)
		
		l<-readLines(f,n=1)
		substrings<-strsplit(l,split=" ")
		num_string<-substrings[[1]][2]
		activation<-as.numeric(num_string)
		#tmpstring<-sprintf("activation: %f",activation)
		#print(tmpstring)
		
		l<-readLines(f,n=1)
		substrings<-strsplit(l,split=" ")
		num_string<-substrings[[1]][2]
		match_value<-as.numeric(num_string)
		#tmpstring<-sprintf("match_value: %f",match_value)
		#print(tmpstring)
		
		l<-readLines(f,n=1)
		substrings<-strsplit(l,split=" ")
		num_string<-substrings[[1]][2]
		edge_num<-as.numeric(num_string)
		#tmpstring<-sprintf("edge_num: %i",edge_num)
		#print(tmpstring)
		
		l<-readLines(f,n=1)
		
		i<-1
		edges<-vector(mode="list",length=0)	
		while(i<=edge_num) {
			edges[[i]]<-new("TopoARTEdge",f,i)
			i<-i+1
		}
		
		l<-readLines(f,n=1)
		
		weights<-vector(mode="numeric",length=dim)
		l<-readLines(f,n=1)
		substrings<-strsplit(l,split=" ")
		i<-1;
		while(i<1+dim) {
			weights[i]<-as.numeric(substrings[[1]][i])
			i<-i+1
		}
		#print(weights)
		
		.Object@ID<-ID
		.Object@cluster_ID<-cluster_ID
		.Object@dim<-dim
		.Object@represented_inputs<-represented_inputs
		.Object@activation<-activation
		.Object@match_value<-match_value
		.Object@edge_num<-edge_num
		.Object@edges<-edges
		.Object@weights<-weights
		.Object
	})
	
#--------------------------------------------------------------------------------------------------
	
	setClass("TopoARTModule",
		representation(
			x_F1_dim="numeric",
			x_F1="vector",
			rho="numeric",
			next_ID="numeric",
			valid_cluster_IDs="logical",
			cluster_num="numeric",
			LearningCycles="numeric",
			node_num="numeric",
			nodes="vector"
		),
		prototype(
			x_F1_dim=0,
			x_F1=vector(mode="numeric",length=0),
			rho=0,
			next_ID=0,
			valid_cluster_IDs=FALSE,
			cluster_num=0,
			LearningCycles=0,
			node_num=0,
			nodes=vector(mode="list",length=0)
		)
	)
	
	setMethod("initialize","TopoARTModule", function(.Object,src_file,module_index)
	{
		letters<-c("a","b","c","d","e")
		tmpstring<-sprintf("load TopoART module %s",letters[module_index])
		print(tmpstring)
		
		f=src_file
		l<-readLines(f,n=3)
		
		l<-readLines(f,n=1)
		substrings<-strsplit(l,split=" ")
		num_string<-substrings[[1]][2]
		x_F1_dim<-as.numeric(num_string)
		#tmpstring<-sprintf("x_F1_dim: %i",x_F1_dim)
		#print(tmpstring)
		
		x_F1<-vector(mode="numeric",length=x_F1_dim)
		l<-readLines(f,n=1)
		substrings<-strsplit(l,split=" ")
		i<-2;
		while(i<2+x_F1_dim) {
			x_F1[i-1]<-substrings[[1]][i]
			i<-i+1
		}
		#print("x_F1:")
		#print(x_F1)
		
		l<-readLines(f,n=1)
		substrings<-strsplit(l,split=" ")
		num_string<-substrings[[1]][2]
		rho<-as.numeric(num_string)
		#tmpstring<-sprintf("rho: %f",rho)
		#print(tmpstring)
		
		l<-readLines(f,n=1)
		substrings<-strsplit(l,split=" ")
		num_string<-substrings[[1]][2]
		next_ID<-as.numeric(num_string)
		#tmpstring<-sprintf("next_ID: %i",next_ID)
		#print(tmpstring)
		
		l<-readLines(f,n=1)
		substrings<-strsplit(l,split=" ")
		logical_string<-substrings[[1]][2]
		valid_cluster_IDs<-(logical_string=="True")
		if(valid_cluster_IDs) {
			tmpstring2<-"TRUE"
		} else {
			tmpstring2<-"FALSE"
		}
		#tmpstring<-sprintf("valid_cluster_IDs: %s",tmpstring2)
		#print(tmpstring)
		
		l<-readLines(f,n=1)
		substrings<-strsplit(l,split=" ")
		num_string<-substrings[[1]][2]
		cluster_num<-as.numeric(num_string)
		#tmpstring<-sprintf("cluster_num: %i",cluster_num)
		#print(tmpstring)
		
		l<-readLines(f,n=1)
		substrings<-strsplit(l,split=" ")
		num_string<-substrings[[1]][2]
		LearningCycles<-as.numeric(num_string)
		#tmpstring<-sprintf("LearningCycles: %i",LearningCycles)
		#print(tmpstring)
		
		l<-readLines(f,n=1)
		substrings<-strsplit(l,split=" ")
		num_string<-substrings[[1]][2]
		node_num<-as.numeric(num_string)
		#tmpstring<-sprintf("node_num: %i",node_num)
		#print(tmpstring)
		
		i<-1
		nodes<-vector(mode="list",length=0)	
		while(i<=node_num) {
			nodes[[i]]<-new("TopoARTNode",f,i)
			i<-i+1
		}
		
		.Object@x_F1_dim<-x_F1_dim
		.Object@x_F1<-x_F1
		.Object@rho<-rho
		.Object@next_ID<-next_ID
		.Object@valid_cluster_IDs<-valid_cluster_IDs
		.Object@cluster_num<-cluster_num
		.Object@LearningCycles<-LearningCycles
		.Object@node_num<-node_num
		.Object@nodes<-nodes
		.Object
	})
	
#--------------------------------------------------------------------------------------------------
	
	setClass("TopoARTNetwork", 
		representation(
			LibTopoART_version="numeric",
			file_format_version="numeric",
			x_F0_dim="numeric",
			x_F0="vector",
			Rho_a="numeric",
			Beta_sbm="numeric",
			Tau="numeric",
			Phi="numeric",
			LearningSteps="numeric",
			Alpha="numeric",
			ModuleNum="numeric",
			modules="vector"
		), 
		prototype(
			LibTopoART_version=0,
			file_format_version=0,
			x_F0_dim=0,
			x_F0=vector(mode="numeric",length=0),
			Rho_a=0,
			Beta_sbm=0,
			Tau=0,
			Phi=0,
			LearningSteps=0,
			Alpha=0,
			ModuleNum=0,
			modules=vector(mode="list",length=0)			
		)
	)
	
	setMethod("initialize","TopoARTNetwork",function(.Object,path)
	{
		tmpstring<-sprintf("load TopoART network: %s",path)
		print(tmpstring)
		
		f<-file(path,"r")
		l<-readLines(f,n=1)
		l<-readLines(f,n=1)
		if(!(l=="*        TopoART network        *")) {
			warning("No TopoART text file")
			close(f)
			return(.Object)
		}
		
		l<-readLines(f,n=1)
		substrings<-strsplit(l,split="v")
		version_string<-substr(substrings[[1]][2],1,4)
		LibTopoART_version<-as.numeric(version_string)
		tmpstring<-sprintf("LibTopoART v%.2f",LibTopoART_version)
		print(tmpstring)
		l<-readLines(f,n=1)
		
		l<-readLines(f,n=1)
		substrings<-strsplit(l,split=" ")
		version_string<-substr(substrings[[1]][2],1,4)
		file_format_version<-as.numeric(version_string)
		#tmpstring<-sprintf("file_format_version: %.2f",file_format_version)
		#print(tmpstring)
		if(file_format_version!=0.09) {
			warning("File format version not supported")
			close(f)
			return(.Object)
		}
		
		l<-readLines(f,n=3)
		l<-readLines(f,n=1)
		substrings<-strsplit(l,split=" ")
		num_string<-substrings[[1]][2]
		x_F0_dim<-as.numeric(num_string)
		#tmpstring<-sprintf("x_F0_dim: %i",x_F0_dim)
		#print(tmpstring)
		
		x_F0<-vector(mode="numeric",length=x_F0_dim)
		l<-readLines(f,n=1)
		substrings<-strsplit(l,split=" ")
		i<-2;
		while(i<2+x_F0_dim) {
			x_F0[i-1]<-substrings[[1]][i]
			i<-i+1
		}
		#print("x_F0:")
		#print(x_F0)
		
		l<-readLines(f,n=1)
		substrings<-strsplit(l,split=" ")
		num_string<-substrings[[1]][2]
		Rho_a<-as.numeric(num_string)
		#tmpstring<-sprintf("Rho_a: %f",Rho_a)
		#print(tmpstring)
		
		l<-readLines(f,n=1)
		substrings<-strsplit(l,split=" ")
		num_string<-substrings[[1]][2]
		Beta_sbm<-as.numeric(num_string)
		#tmpstring<-sprintf("Beta_sbm: %f",Beta_sbm)
		#print(tmpstring)
		
		l<-readLines(f,n=1)
		substrings<-strsplit(l,split=" ")
		num_string<-substrings[[1]][2]
		Tau<-as.numeric(num_string)
		#tmpstring<-sprintf("Tau: %i",Tau)
		#print(tmpstring)
		
		l<-readLines(f,n=1)
		substrings<-strsplit(l,split=" ")
		num_string<-substrings[[1]][2]
		Phi<-as.numeric(num_string)
		#tmpstring<-sprintf("Phi: %i",Phi)
		#print(tmpstring)
		
		l<-readLines(f,n=1)
		substrings<-strsplit(l,split=" ")
		num_string<-substrings[[1]][2]
		LearningSteps<-as.numeric(num_string)
		#tmpstring<-sprintf("LearningSteps: %i",LearningSteps)
		#print(tmpstring)
		
		l<-readLines(f,n=1)
		substrings<-strsplit(l,split=" ")
		num_string<-substrings[[1]][2]
		Alpha<-as.numeric(num_string)
		#tmpstring<-sprintf("Alpha: %f",Alpha)
		#print(tmpstring)
		
		l<-readLines(f,n=1)
		substrings<-strsplit(l,split=" ")
		num_string<-substrings[[1]][2]
		ModuleNum<-as.numeric(num_string)
		#tmpstring<-sprintf("ModuleNum: %i",ModuleNum)
		#print(tmpstring)
		
		i<-1
		modules<-vector(mode="list",length=0)	
		while(i<=ModuleNum) {
			modules[[i]]<-new("TopoARTModule",f,i)
			i<-i+1
		}
		
		close(f)
		
		.Object@LibTopoART_version<-LibTopoART_version
		.Object@file_format_version<-file_format_version
		.Object@x_F0_dim<-x_F0_dim
		.Object@x_F0<-x_F0
		.Object@Rho_a<-Rho_a
		.Object@Beta_sbm<-Beta_sbm
		.Object@Tau<-Tau
		.Object@Phi<-Phi
		.Object@LearningSteps<-LearningSteps
		.Object@Alpha<-Alpha
		.Object@ModuleNum<-ModuleNum
		.Object@modules<-modules
		.Object
	})
	
	setGeneric("SaveTopoARTPlot", function(this) standardGeneric("SaveTopoARTPlot"))
	setMethod("SaveTopoARTPlot","TopoARTNetwork",function(this)
	{
		ColorShift<-4
		MaxClusterNum<-this@modules[[this@ModuleNum]]@cluster_num
		letters<-c("a","b","c","d","e")
		
		m<-1
		while(m<=this@ModuleNum) {
			tmpstring=sprintf("images/TA_%s.pdf",letters[m])
			pdf(file=tmpstring)
			palette(rainbow(MaxClusterNum))
			par(cex.axis=1.5)
			par(cex.main=1.6)
			plot(NULL,NULL,xlim=c(0,1),ylim=c(0,1),xlab="",ylab="")
			grid()
			tmpstring=sprintf("TA %s: Rho_a=%.2f, Beta_sbm=%.2f, Phi=%i, Tau=%i",letters[m],this@Rho_a,this@Beta_sbm,this@Phi,this@Tau)
			title(main=tmpstring, font.main=2)

			n<-1;
			while(n<=this@modules[[m]]@node_num) {
				x1<-this@modules[[m]]@nodes[[n]]@weights[[1]]
				y1<-this@modules[[m]]@nodes[[n]]@weights[[2]]
				x2<-1-this@modules[[m]]@nodes[[n]]@weights[[3]]
				y2<-1-this@modules[[m]]@nodes[[n]]@weights[[4]]
				if(this@modules[[m]]@nodes[[n]]@cluster_ID>0) {
					rect(x1,y1,x2,y2,col=(this@modules[[m]]@nodes[[n]]@cluster_ID-1+ColorShift)%%MaxClusterNum+1)	
				}
				n<-n+1
			}
			
			m<-m+1
			dev.off()
		}
	})	
}
